package com.bweb.controller;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.utils.HttpUtils;
import com.hfkj.common.utils.HttpsUtils;
import com.hfkj.common.utils.ResponseMsgUtil;
import com.hfkj.entity.SecMenu;
import com.hfkj.entity.SecRoleMenuRel;
import com.hfkj.model.MenuTreeModel;
import com.hfkj.model.ResponseData;
import com.hfkj.service.SecMenuService;
import com.hfkj.service.SecRoleMenuRelService;
import com.hfkj.sysenum.SecMenuTypeEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpResponse;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @className: SecMenu
 * @author: HuRui
 * @date: 2024/3/28
 **/
@Controller
@RequestMapping(value="/secMenu")
@Api(value="系统菜单管理")
public class SecMenuController {

    Logger log = LoggerFactory.getLogger(SecUserController.class);

    @Resource
    private SecMenuService secMenuService;
    @Resource
    private SecRoleMenuRelService secRoleMenuRelService;

    @RequestMapping(value="/editMenu",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "编辑菜单")
    public ResponseData editMenu(@RequestBody SecMenu body) {
        try {
            if (body == null
                    || body.getMenuType() == null
                    || StringUtils.isBlank(body.getMenuName())
                    || StringUtils.isBlank(body.getMenuUrl())
                    || body.getMenuSort() == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }
            SecMenu secMenu;

            if (body.getId() != null) {
                // 查询菜单
                secMenu = secMenuService.queryDetail(body.getId());
                if (secMenu == null) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
                }
            } else {
                secMenu = new SecMenu();
            }
            secMenu.setMenuType(body.getMenuType());
            secMenu.setMenuName(body.getMenuName());
            secMenu.setMenuUrl(body.getMenuUrl());
            secMenu.setMenuUrlImg(body.getMenuUrlImg());
            secMenu.setMenuPSid(body.getMenuPSid());
            secMenu.setMenuSort(body.getMenuSort());
            secMenu.setMenuDesc(body.getMenuDesc());
            if (secMenu.getId() == null) {
                secMenuService.create(secMenu);
            } else {
                secMenuService.update(secMenu);
            }
            return ResponseMsgUtil.success("操作成功");

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryDetail",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "查询详情")
    public ResponseData queryDetail(@RequestParam(value = "menuId" , required = true) Long menuId) {
        try {

            return ResponseMsgUtil.success(secMenuService.queryDetail(menuId));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/assignMenu",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "分配菜单")
    public ResponseData assignMenu(@RequestBody JSONObject body) {
        try {
            if (body == null
                    || body.getLong("roleId") == null
                    || body.getJSONArray("menuIds").isEmpty()) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            List<Long> list = body.getJSONArray("menuIds")
                    .stream().map(o -> Long.parseLong(o.toString()))
                    .collect(Collectors.toList());

            secMenuService.assignMenu(body.getLong("roleId"),list );
            return ResponseMsgUtil.success("操作成功");

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryMenuList",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询菜单列表")
    public ResponseData queryMenuList() {
        try {

            return ResponseMsgUtil.success(secMenuService.getAllList());

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryAssignMenuTree",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询分配菜单树")
    public ResponseData queryAssignMenuTree(@RequestParam(value = "roleId" , required = true) Long roleId) {
        try {

            List<MenuTreeModel> treeModelList = new ArrayList<>();
            MenuTreeModel menuTree;

            // 获取系统菜单
            List<SecMenu> sysMenuList = secMenuService.getAllList();
            Map<Long, SecRoleMenuRel> roleMenuRelMap = secRoleMenuRelService.getRelListByRole(roleId)
                    .stream()
                    .collect(Collectors.toMap(SecRoleMenuRel::getMenuId, Function.identity()));

            // 获取系统顶级菜单
            List<SecMenu> topLevelMenuList = sysMenuList.stream().filter(o -> o.getMenuPSid() == null)
                    .sorted(Comparator.comparing(SecMenu::getMenuSort))
                    .collect(Collectors.toList());

            for (SecMenu topLevelMenu : topLevelMenuList) {
                menuTree = new MenuTreeModel();
                menuTree.setId(topLevelMenu.getId());
                menuTree.setMenuName(topLevelMenu.getMenuName());
                menuTree.setMenuType(topLevelMenu.getMenuType());
                menuTree.setMenuUrl(topLevelMenu.getMenuUrl());
                menuTree.setMenuUrlImg(topLevelMenu.getMenuUrlImg());
                menuTree.setMenuPSid(topLevelMenu.getMenuPSid());
                menuTree.setMenuSort(topLevelMenu.getMenuSort());
                menuTree.setMenuDesc(topLevelMenu.getMenuDesc());
                // 递归子菜单
                menuTree.setChildMenuList(recursionMenu(sysMenuList, roleMenuRelMap, topLevelMenu.getId()));
                // 获取角色菜单
                menuTree.setAuthority(roleMenuRelMap.get(topLevelMenu.getId()) != null ? true : false);
                treeModelList.add(menuTree);
            }

            return ResponseMsgUtil.success(treeModelList);

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryRoleMenuTree",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询角色菜单树")
    public ResponseData queryRoleMenuTree(@RequestParam(value = "roleId" , required = false) Long roleId) {
        try {

            return ResponseMsgUtil.success(secMenuService.queryMenuTree(roleId));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    /**
     * 递归菜单
     * @param sysMenu 系统菜单
     * @param roleMenuRel  角色菜单
     * @param parentMenuId 父类菜单id
     * @return
     */
    public List<MenuTreeModel> recursionMenu(List<SecMenu> sysMenu, Map<Long, SecRoleMenuRel> roleMenuRel, Long parentMenuId) {
        List<MenuTreeModel> treeModelList = new ArrayList<>();
        MenuTreeModel menuTree;

        List<SecMenu> collect = sysMenu.stream()
                .filter(o -> o.getMenuPSid() != null && o.getMenuPSid().equals(parentMenuId))
                .sorted(Comparator.comparing(SecMenu::getMenuSort))
                .collect(Collectors.toList());
        for (SecMenu menu : collect) {
            menuTree = new MenuTreeModel();
            menuTree.setId(menu.getId());
            menuTree.setMenuName(menu.getMenuName());
            menuTree.setMenuType(menu.getMenuType());
            menuTree.setMenuUrl(menu.getMenuUrl());
            menuTree.setMenuUrlImg(menu.getMenuUrlImg());
            menuTree.setMenuPSid(menu.getMenuPSid());
            menuTree.setMenuSort(menu.getMenuSort());
            menuTree.setMenuDesc(menu.getMenuDesc());
            // 获取角色菜单
            menuTree.setAuthority(roleMenuRel.get(menuTree.getId()) != null ? true : false);
            menuTree.setChildMenuList(recursionMenu(sysMenu, roleMenuRel, menu.getId()));
            treeModelList.add(menuTree);
        }
        return treeModelList;
    }
}
